from NMIXStudio.NMIImporter.BuiltInImporter import*
from NMIXStudio.NMIImporter.Decarators import*
from NMIXStudio.RequestManager.NMIEndpoint import NMIEndpoint
from NMIXStudio.RequestManager.NMICommand import NMICommand

PREFIX = "XYOffset"
class GET_Commands(Enum):
    Get_Commands                = "Get_Commands"
    Get_IsOffsetUpdateAvailable = "Get_IsOffsetUpdateAvailable"
    Get_XOffset                 = "Get_XOffset"
    Get_YOffset                 = "Get_YOffset"
    Get_XYOffset                = "Get_XYOffset"
    Get_XOffsetLimit            = "Get_XOffsetLimit"
    Get_YOffsetLimit            = "Get_YOffsetLimit"
    Get_XYOffsetLimit           = "Get_XYOffsetLimit"

class SET_Commands(Enum):
    Set_XOffset         = "Set_XOffset"
    Set_YOffset         = "Set_YOffset"
    Set_XYOffset        = "Set_XYOffset"
    Set_DirectXOffset   = "Set_DirectXOffset"
    Set_DirectYOffset   = "Set_DirectYOffset"

    Start_GuiOffsetUpdater = "Start_GuiOffsetUpdater"
    Stop_GuiOffsetUpdater  = "Stop_GuiOffsetUpdater"

    

class XYOffsetController:
    def __init__(self,endpoint:NMIEndpoint) -> None:
        self.Endpoint = endpoint

    @ExceptionLoggerAPIResponse
    def Get_Commands(self)->list:
        command = NMICommand(self.Endpoint,PREFIX,GET_Commands.Get_Commands.value)
        try:
            response = command.execute_get()
            if("Commands" in response):
                return response["Commands"].split(';')
            else:
                return []
        except Exception as e:
            return []

    @ExceptionLoggerAPIResponse
    def Get_IsOffsetUpdateAvailable(self)->dict:
        command = NMICommand(self.Endpoint,PREFIX,GET_Commands.Get_IsOffsetUpdateAvailable.value)
        response = command.execute_get()
        return response

    
    @ExceptionLoggerAPIResponse
    def Get_XOffset(self)->dict:
        command = NMICommand(self.Endpoint,PREFIX,GET_Commands.Get_XOffset.value)
        response = command.execute_get()
        return response

    @ExceptionLoggerAPIResponse
    def Get_YOffset(self)->dict:
        command = NMICommand(self.Endpoint,PREFIX,GET_Commands.Get_YOffset.value)
        response = command.execute_get()
        return response

    @ExceptionLoggerAPIResponse
    def Get_XYOffset(self)->dict:
        command = NMICommand(self.Endpoint,PREFIX,GET_Commands.Get_XYOffset.value)
        response = command.execute_get()
        return response

    @ExceptionLoggerAPIResponse
    def Get_XOffsetLimit(self)->dict:
        command = NMICommand(self.Endpoint,PREFIX,GET_Commands.Get_XOffsetLimit.value)
        response = command.execute_get()
        return response

    @ExceptionLoggerAPIResponse
    def Get_YOffsetLimit(self)->dict:
        command = NMICommand(self.Endpoint,PREFIX,GET_Commands.Get_YOffsetLimit.value)
        response = command.execute_get()
        return response

    @ExceptionLoggerAPIResponse
    def Get_XYOffsetLimit(self)->dict:
        command = NMICommand(self.Endpoint,PREFIX,GET_Commands.Get_XYOffsetLimit.value)
        response = command.execute_get()
        return response

    @ExceptionLoggerAPIResponse
    def Set_XOffset(self,xOffset:float)->dict:
        payload = {
            'reg0': float(xOffset),
            'reg1': "",
            'reg2': "",
            'reg3': "",
        }
        command = NMICommand(self.Endpoint,PREFIX,SET_Commands.Set_XOffset.value,payload)
        response = command.execute_post()
        return response

    @ExceptionLoggerAPIResponse
    def Set_YOffset(self,yOffset:float)->dict:
        payload = {
            'reg0': float(yOffset),
            'reg1': "",
            'reg2': "",
            'reg3': "",
        }
        command = NMICommand(self.Endpoint,PREFIX,SET_Commands.Set_YOffset.value,payload)
        response = command.execute_post()
        return response
    
    @ExceptionLoggerAPIResponse
    def Set_DirectXOffset(self,yOffset:float)->dict:
        payload = {
            'reg0': float(yOffset),
            'reg1': "",
            'reg2': "",
            'reg3': "",
        }
        command = NMICommand(self.Endpoint,PREFIX,SET_Commands.Set_DirectXOffset.value,payload)
        response = command.execute_post()
        return response
    
    @ExceptionLoggerAPIResponse
    def Set_DirectYOffset(self,yOffset:float)->dict:
        payload = {
            'reg0': float(yOffset),
            'reg1': "",
            'reg2': "",
            'reg3': "",
        }
        command = NMICommand(self.Endpoint,PREFIX,SET_Commands.Set_DirectYOffset.value,payload)
        response = command.execute_post()
        return response

    @ExceptionLoggerAPIResponse
    def Set_XYOffset(self,xOffset:float,yOffset:float)->dict:
        payload = {
            'reg0': float(xOffset),
            'reg1': float(yOffset),
            'reg2': "",
            'reg3': "",
        }
        command = NMICommand(self.Endpoint,PREFIX,SET_Commands.Set_XYOffset.value,payload)
        response = command.execute_post()
        return response

    @ExceptionLoggerAPIResponse
    def Start_GuiOffsetUpdater(self)->dict:
        command = NMICommand(self.Endpoint,PREFIX,SET_Commands.Start_GuiOffsetUpdater.value)
        response = command.execute_post()
        return response

    @ExceptionLoggerAPIResponse
    def Stop_GuiOffsetUpdater(self)->dict:
        command = NMICommand(self.Endpoint,PREFIX,SET_Commands.Stop_GuiOffsetUpdater.value)
        response = command.execute_post()
        return response


from NMIXStudio._version import __version__
from setuptools import setup, find_packages

# requirements.txt dosyasını okuyarak bağımlılıkları al
with open("requirements.txt", "r") as f:
    requirements = f.read().splitlines()

setup(
    name="nmixstudio",
    version=__version__,
    author="NanoMagnetics Instruments",
    author_email="nmi.swteam@nano.com.tr",
    description="",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=requirements,
    include_package_data=True,  # Bu satır eklenmeli
    package_data={
        "": ["NMIXStudio/Docs/*", "NMIXStudio/Samples/*"],  # Belirli dosya ve klasörleri dahil eder
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)


from NMIXStudio.Container.NmiImage.image import NMIImage
from NMIXStudio.Container.NmiContainer.enums import*
from NMIXStudio.Container.NmiFrame.frameChannels import*
class NMIImageBuilder():
    def __init__(self):
        self.Image = NMIImage()
        pass
    
    def SetWidthPixe(self,width):
        self.Image.WidthPixel = width
        return self

    def SetHeightPixel(self,height):
        self.Image.HeightPixel = height
        return self

    def SetRealHeight(self,height):
        self.Image.RealHeight = height
        return self
    
    def SetRealWidth(self,width):
        self.Image.RealWidth = width
        return self
    
    def SetRealHeightUnit(self,unit:ScaleUnit):
        self.Image.RealHeightUnit = unit
        return self
    
    def SetRealWidthUnit(self,unit:ScaleUnit):
        self.Image.RealWidthUnit = unit
        return self

    def SetRealHeightUnitPrefix(self,prefix:ScalePrefix):
        self.Image.RealHeightUnitPrefix = prefix
        return self
    
    def SetRealWidthUnitPrefix(self,prefix:ScalePrefix):
        self.Image.RealWidthUnitPrefix = prefix
        return self
    
    def SetRealBufferUnit(self,unit:ScaleUnit):
        self.Image.RealBufferUnit = unit
        return self
    def SetRealBufferUnitPrefix(self,prefix:ScalePrefix):
        self.Image.RealBufferUnitPrefix = prefix
        return self
    
    def SetChannel(self,channel:FrameChannels):
        self.Image.channel = channel
        self.Image.imageName = channel.name
        return self
        
    def Build(self)->NMIImage:
        self.Image.CreateNewFrame()
        return self.Image



import re
from urllib.parse import urlparse

class NMIEndpoint:
    def __init__(self, ip: str, port: int):
        self.ip = ip
        self.port = port

    def __str__(self):
        return f"Endpoint(IP: {self.ip}, Port: {self.port})"

    def __repr__(self):
        return f"Endpoint(ip='{self.ip}', port={self.port})"

    def is_valid(self) -> bool:
        return self.is_valid_ip() and self.is_valid_port()

    def is_valid_ip(self) -> bool:
        import re
        pattern = re.compile(r'^(\d{1,3}\.){3}\d{1,3}$')
        if not pattern.match(self.ip):
            return False
        
        parts = self.ip.split('.')
        return all(0 <= int(part) < 256 for part in parts)

    def is_valid_port(self) -> bool:
        return 0 <= self.port <= 65535
    
    @classmethod
    def from_url(cls, url: str):
        parsed_url = urlparse(url)
        host = parsed_url.hostname
        port = parsed_url.port
        
        if not host or port is None:
            raise ValueError("Geçersiz URL: IP adresi veya port bulunamadı.")
        
        if not cls.is_valid_ip_address(host):
            raise ValueError("Geçersiz IP adresi.")
        
        if not (0 <= port <= 65535):
            raise ValueError("Geçersiz port numarası.")
        
        return cls(ip=host, port=port)
    
    @staticmethod
    def is_valid_ip_address(ip: str) -> bool:
        pattern = re.compile(r'^(\d{1,3}\.){3}\d{1,3}$')
        if not pattern.match(ip):
            return False
        
        parts = ip.split('.')
        return all(0 <= int(part) < 256 for part in parts)

    def to_string(self) -> str:
        return f"http://{self.ip}:{self.port}/"


from NMIXStudio.NMIImporter.BuiltInImporter import*
from NMIXStudio.NMIImporter.Decarators import*
from NMIXStudio.RequestManager.NMIEndpoint import NMIEndpoint
from NMIXStudio.RequestManager.NMICommand import NMICommand

PREFIX = "ScannedImages"

class GET_Commands(Enum):
    Get_Commands = "Get_Commands"
    Get_NmiContainerList = "Get_NmiContainerList"
    Get_SelectedContainerImageList = "Get_SelectedContainerImageList"
    Get_SelectedContainerForwardImageList = "Get_SelectedContainerForwardImageList"
    Get_SelectedContainerBackwardImageList = "Get_SelectedContainerBackwardImageList"
    Get_SelectedContainerImage = "Get_SelectedContainerImage"

class SET_Commands(Enum):
    pass

class ScannedImagesController:
    def __init__(self,endpoint:NMIEndpoint) -> None:
        self.Endpoint = endpoint

    def Get_Commands(self)->list:
        command = NMICommand(self.Endpoint,PREFIX,GET_Commands.Get_Commands.value)
        response = command.execute_get()
        if("Commands" in response):
            return response["Commands"].split(';')
        else:
            return []
        
    @ExceptionLoggerAPIResponse
    def Get_NmiContainers(self)->list:
        command = NMICommand(self.Endpoint,PREFIX,GET_Commands.Get_NmiContainerList.value)
        response = command.execute_get()
        if("NmiContainerList" in response):
            return response["NmiContainerList"].split(';')
        else:
            return []
    
    @ExceptionLoggerAPIResponse
    def Get_SelectedContainerImageList(self,containerName:str)->list:
        payload = {
            'reg0': str(containerName),
            'reg1': "",
            'reg2': "",
            'reg3': "",
        }
        command = NMICommand(self.Endpoint,PREFIX,GET_Commands.Get_SelectedContainerImageList.value,payload)
        response = command.execute_post()
        if("SelectedContainerImageList" in response):
            return response["SelectedContainerImageList"].split(';')
        else:
            return []

    @ExceptionLoggerAPIResponse   
    def Get_SelectedContainerForwardImageList(self,containerName:str)->list:
        payload = {
            'reg0': str(containerName),
            'reg1': "",
            'reg2': "",
            'reg3': "",
        }
        command = NMICommand(self.Endpoint,PREFIX,GET_Commands.Get_SelectedContainerForwardImageList.value,payload)
        response = command.execute_post()
        if("SelectedContainerForwardImageList" in response):
            return response["SelectedContainerForwardImageList"].split(';')
        else:
            return []
    
    @ExceptionLoggerAPIResponse
    def Get_SelectedContainerBackwardImageList(self,containerName:str)->list:
        payload = {
            'reg0': str(containerName),
            'reg1': "",
            'reg2': "",
            'reg3': "",
        }
        command = NMICommand(self.Endpoint,PREFIX,GET_Commands.Get_SelectedContainerBackwardImageList.value,payload)
        response = command.execute_post()
        if("SelectedContainerBackwardImageList" in response):
            return response["SelectedContainerBackwardImageList"].split(';')
        else:
            return []

    @ExceptionLoggerAPIResponse  
    def Get_SelectedContainerImage(self,containerName:str,imageName:str)->dict:
        payload = {
            'reg0': str(containerName),
            'reg1': str(imageName),
            'reg2': "",
            'reg3': "",
        }
        command = NMICommand(self.Endpoint,PREFIX,GET_Commands.Get_SelectedContainerImage.value,payload)
        response = command.execute_post()
        return response


from NMIXStudio import NMICommand,NMIEndpoint,NMIDevice
import time

if __name__ == "__main__":
    # Define the endpoint by specifying the device's IP address and port
    # Ensure the IP and port match the device configuration
    Endpoint = NMIEndpoint("192.168.10.31",9024)

    # Initialize the device with the specified endpoint 
    Device = NMIDevice(Endpoint)

    # Perform basic device operations
    Options = Device.OPTIONS()
    XYOffsetController = Device.XYOFFSET()
    Status = Device.STATUS()

   
    Scan = Device.SCAN()

    # Retrieve current scale settings
    print(Options.Get_XYScale())  # Get XY scale in μm
    print(Options.Get_ZScale())  # Get Z scale in nm

    # Set initial XY offset position for scan
    XYOffsetController.Set_XYOffset(0,0)  # Update XY offset to (0, 0)

    # Set scan parameters
    print(Scan.Set_XOffset(0))  # Update X offset to 0
    print(Scan.Set_YOffset(0))  # Update Y offset to 0
    print(Scan.Set_ScanHeightPixel(64))  # Update scan height to 64 pixels
    print(Scan.Set_ScanWidthPixel(64))  # Update scan width to 64 pixels
    print(Scan.Set_ImageWidth(3))  # Update image width to 3 μm
    print(Scan.Set_ImageHeight(3))  # Update image height to 3 μm
    print(Scan.Set_ScanAngle(0))  # Update scan angle to 0 degrees
    print(Scan.Set_ScanNumberOfAverages(4))  # Update averages to 4
    print(Scan.Set_NumberOfScans(1))  # Update total scans to 1
    print(Scan.Set_ScanSpeed(5))  # Update scan speed to 5
    print(Scan.Set_IsSaveScannedImages(True))  # Enable image saving
    print(Scan.Set_OffsetPosition("BottomLeft"))  # Update position to bottom-left
    print(Scan.Set_ScanDirection("BottomToTop"))  # Update scan direction to bottom-to-top
    print(Scan.Set_IsImageSquare(True))  # Update as enables or disables the square shape
    print(Scan.Set_IsRoundtripScan(True))  # Update as enable or disable the roundtrip scan

    # Retrieve and print scan-related information (current offset in μm, dimensions in pixels, etc.)
    print(Scan.Get_Commands())  # List available scan commands
    print(Scan.Get_IsScanning())  # Is scan currently in progress?
    print(Scan.Get_ScanError())  # Current scan error status
    print(Scan.Get_ScanLineIndex())  # Current scan line index
    print(Scan.Get_ScanIndex())  # Current scan index
    print(Scan.Get_XOffset())  # X offset
    print(Scan.Get_YOffset())  # Y offset
    print(Scan.Get_ScanWidthPixel())  # Scan width in pixels
    print(Scan.Get_ScanHeightPixel())  # Scan height in pixels
    print(Scan.Get_ImageWidth())  # Real image width
    print(Scan.Get_ImageHeight())  # Real image height
    print(Scan.Get_ScanAngle())  # Scan angle
    print(Scan.Get_ScanSpeed())  # Scan speed
    print(Scan.Get_ScanNumberOfAverages())  # Number of averages
    print(Scan.Get_NumberOfScans())  # Total number of scans
    print(Scan.Get_OffsetPosition())  # Offset position
    print(Scan.Get_ScanDirection())  # Scan direction
    print(Scan.Get_IsRoundtripScan())  # Is roundtrip scan enabled?
    print(Scan.Get_IsSaveScannedImages())  # Is saving scanned images enabled?
    print(Scan.Get_IsImageSquare())  # Is the scanned image square?
    
    print(Scan.Get_CapturePixel())  # Get the capture pixel value for the scan
    print(Scan.Get_CapturePixelUnitText())  # Get the unit text for the capture pixel (e.g., "pixels")
    
    # Check scanning status and errors
    isScanning = (bool)(Scan.Get_IsScanning()["IsScanning"])  # Is scanning in progress?
    hasError = Scan.Get_ScanError()["ScanError"]  # Retrieve scan error status
    status = Status.Get_Status()  # Retrieve current device status
    time.sleep(2)

    # Start scanning process if ready
    if(isScanning==False and status == "Ready"):
        Scan.StartScan(True)  # Start scan process
        time.sleep(2)
        while(True):
            isScanning = (bool)(Scan.Get_IsScanning()["IsScanning"])
            if(isScanning==False):
                break   # Exit loop if scanning is complete
            print(Scan.Get_ScanLineIndex())  # Current scan line index
            print(Scan.Get_ScanIndex())  # Current scan index
            time.sleep(0.5)
    
    # Stop scanning
    Scan.StopScan(True)
    
    # Check for any errors after scanning
    hasError = Scan.Get_ScanError()["ScanError"]
    print(hasError)


 


from .utils import ExternalScanDevice
import pyvisa
import time
import logging
from enum import Enum

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)

# -----------------------------------------------------
# Enum Definitions
# -----------------------------------------------------
class SPCM50_OPERATINGMODES(Enum):
    Null = -1
    Idle = 0
    Manual = 1
    FreeRunning = 2
    ExternallyTriggeredTimed = 3
    ExternallyTriggeredCount = 4
    ExternalGating = 5

class SPCM50_GATINGMODE(Enum):
    OFF = 0
    ON = 1

class PhotonCounterDevices(Enum):
    SPCM50A = 0

# -----------------------------------------------------
# SPCM50ControlModel Class
# -----------------------------------------------------
class SPCM50ControlModel(ExternalScanDevice):
    """
    Example Python class for communicating with a photon counter such as the Thorlabs SPCM50 via VISA.
    """

    # ---------------------------
    # Default Values
    # ---------------------------
    DEFAULT_BIN_LENGTH = 20.0          # in ms
    DEFAULT_TIME_BETWEEN_DELAY = 0.0   # in ms
    DEFAULT_PULSE_BLIND_TIME = 8.0     # in ms
    DEFAULT_AVERAGE_COUNT = 10
    RESOURCE_ADDRESS_KEYWORD = "0x1313"

    def __init__(self):
        # Example to store properties similar to C# properties
        self.is_initialized = False
        self.is_measurement_running = False

        self.bin_length = self.DEFAULT_BIN_LENGTH
        self.time_between_delay = self.DEFAULT_TIME_BETWEEN_DELAY
        self.pulse_blind_time = self.DEFAULT_PULSE_BLIND_TIME
        self.average_count = self.DEFAULT_AVERAGE_COUNT

        self.last_average_photon_count = 0

        self.session = None
        self.resource_manager = None
        self.visa_resource_address = None
        self.device_name = "-"
        self.curr_device = PhotonCounterDevices.SPCM50A  # Example

        # Variables to simulate UI indicators (similar to C# style)
        self.is_busy_indicator = False
        self.busy_content = "Connecting..."

        # Example simplified implementation for connection control flag
        self._connection_control_flag = False

    # ---------------------------
    # Public Methods
    # ---------------------------
    def load_settings(self):
        """
        Load settings (e.g., self.bin_length, self.time_between_delay) from an external configuration file.
        This could be a JSON or .ini file. Currently using default values.
        """
        self.is_busy_indicator = False
        self.busy_content = "Connecting..."
        logger.info("Settings loaded (using default values).")

    def set_to_default(self):
        """
        Resets the device settings to default values.
        """
        self.is_busy_indicator = False
        self.busy_content = "Connecting..."

        self.bin_length = self.DEFAULT_BIN_LENGTH
        self.time_between_delay = self.DEFAULT_TIME_BETWEEN_DELAY
        self.pulse_blind_time = self.DEFAULT_PULSE_BLIND_TIME
        self.average_count = self.DEFAULT_AVERAGE_COUNT

        logger.info("Settings reset to default values.")

    def initialize(self):
        """
        Finds the device, establishes a VISA connection, and performs basic initial settings.
        """
        try:
            self.is_busy_indicator = True
            self.busy_content = "Connecting..."

            # 1) Create a Resource Manager
            self.resource_manager = pyvisa.ResourceManager()

            # 2) Find appropriate VISA resource
            self.visa_resource_address = self.find_visa_resource_address()
            time.sleep(1)

            # 3) Exit if not found
            if self.visa_resource_address is None:
                logger.error("Device not found.")
                self.is_busy_indicator = False
                self.busy_content = "Connecting..."
                return False

            # 4) Open session
            self.session = self.resource_manager.open_resource(self.visa_resource_address)

            # 5) Connection established, perform basic settings
            self.is_initialized = True

            # 6) Get device name
            self.set_device_name()

            # 7) Reset the device (if it supports a reset command)
            self.reset_device()

            self.is_busy_indicator = False
            self.busy_content = "Connecting..."

            # 8) (Optional) Start monitoring the connection
            self._connection_control_flag = True

            logger.info(f"Device initialized: {self.device_name}")
            return True

        except pyvisa.VisaIOError as ex:
            logger.error(f"VISA connection error: {ex}")
            self.device_name = "-"
            self.is_busy_indicator = False
            self.busy_content = "Connecting..."
            return False

    def reinitialize(self):
        """
        Closes the session and reinitializes the device.
        """
        self.session_dispose()
        return self.initialize()

    def measurement_init(self):
        """
        Initializes settings for measurement (e.g., sets the device to free-running mode, disables gating).
        """
        if not self.is_initialized:
            logger.error("Device is not initialized.")
            return False
        try:
            self.set_operating_mode(SPCM50_OPERATINGMODES.FreeRunning)
            self.set_bin_length(self.bin_length)
            self.set_time_between(self.time_between_delay)
            self.set_gating_mode(SPCM50_GATINGMODE.OFF)
            logger.info("Measurement initialization completed.")
            return True
        except Exception as ex:
            logger.error(f"Measurement initialization error: {ex}")
            return False

    def get_average_count(self):
        """
        Takes the specified number of measurements (average_count) and returns the average value.
        """
        if not self.is_initialized:
            logger.error("Device is not initialized.")
            return 0.0

        self.start_measurement()
        time.sleep((self.time_between_delay + self.bin_length) / 1000.0 + 0.1)

        total_counts = 0.0
        for _ in range(self.average_count):
            count_value_str = self.get_counter_value()
            parsed_value = self._parse_counter_value(count_value_str)
            total_counts += parsed_value

            time.sleep((self.time_between_delay + self.bin_length) / 1000.0 + 0.01)

        self.stop_measurement()
        average_counts = total_counts / self.average_count

        self.last_average_photon_count = average_counts
        logger.info(f"Average photon count: {average_counts}")
        return average_counts

    def start_measurement(self):
        if not self.is_initialized:
            logger.error("Device is not initialized.")
            return False
        try:
            self.send_command("MEASure:STARt")
            self.is_measurement_running = True
            logger.info("Measurement started.")
            return True
        except Exception as ex:
            logger.error(f"StartMeasurement error: {ex}")
            return False

    def stop_measurement(self):
        if not self.is_initialized:
            logger.error("Device is not initialized.")
            return False
        try:
            self.send_command("MEASure:STOP")
            self.is_measurement_running = False
            logger.info("Measurement stopped.")
            return True
        except Exception as ex:
            logger.error(f"StopMeasurement error: {ex}")
            return False

    def session_dispose(self):
        """
        Closes the session (releases VISA resources).
        """
        if self.session is not None:
            try:
                self.session.close()
            except Exception as ex:
                logger.error(f"Session close error: {ex}")
        self.session = None
        self.visa_resource_address = None
        self.is_initialized = False
        logger.info("Session closed.")

    # ---------------------------
    # Private/Helper Methods
    # ---------------------------
    def find_visa_resource_address(self):
        """
        Finds the device with '0x1313' in its resource string.
        """
        try:
            if self.resource_manager is None:
                self.resource_manager = pyvisa.ResourceManager()

            resources = self.resource_manager.list_resources()
            for rsc in resources:
                if self.RESOURCE_ADDRESS_KEYWORD in rsc:
                    return rsc
            return None
        except Exception as ex:
            logger.error(f"VISA resource search error: {ex}")
            return None

    def send_command(self, command: str):
        """
        Sends a command to the device.
        """
        if not self.is_initialized or self.session is None:
            return
        try:
            self.session.write(command)
        except Exception as ex:
            logger.error(f"Command send error: {command}, {ex}")
            self.session_dispose()

    def read_command(self) -> str:
        """
        Reads a response from the device.
        """
        if not self.is_initialized or self.session is None:
            return ""
        try:
            return self.session.read().strip()
        except Exception as ex:
            logger.error(f"Response read error: {ex}")
            self.session_dispose()
            return ""

    def set_device_name(self):
        """
        Reads the device name using *IDN? query and saves it.
        """
        self.send_command("*IDN?")
        response = self.read_command()
        parts = response.split(',')
        if len(parts) >= 2:
            self.device_name = parts[1]
        logger.info(f"Device name: {self.device_name}")

    def reset_device(self):
        """
        Sends a hardware reset command if supported (e.g., *RST).
        """
        logger.info("Device reset (example).")

    def get_counter_value(self) -> str:
        """
        Requests the measurement value (e.g., using a DATA? command).
        """
        if not self.is_initialized:
            return ""
        try:
            self.send_command("DATA?")
            return self.read_command()
        except Exception as ex:
            logger.error(f"Counter value retrieval error: {ex}")
            return ""

    # ---------------------------
    # Configuration and Status Methods
    # ---------------------------
    def set_operating_mode(self, mode: SPCM50_OPERATINGMODES) -> bool:
        if not self.is_initialized:
            return False
        try:
            self.send_command(f"GATE:MODE {mode.value}")
            return True
        except Exception as ex:
            logger.error(f"SetOperatingMode error: {ex}")
            return False

    def get_operating_mode(self) -> SPCM50_OPERATINGMODES:
        if not self.is_initialized:
            return SPCM50_OPERATINGMODES.Null
        try:
            self.send_command("GATE:MODE?")
            mode_str = self.read_command()
            mode_val = int(mode_str)
            return SPCM50_OPERATINGMODES(mode_val)
        except Exception as ex:
            logger.error(f"GetOperatingMode error: {ex}")
            return SPCM50_OPERATINGMODES.Null

    def set_gating_mode(self, mode: SPCM50_GATINGMODE) -> bool:
        if not self.is_initialized:
            return False
        try:
            self.send_command(f"APD:GATE {mode.value}")
            return True
        except Exception as ex:
            logger.error(f"SetGatingMode error: {ex}")
            return False

    def get_gating_mode(self) -> SPCM50_GATINGMODE:
        if not self.is_initialized:
            return SPCM50_GATINGMODE.OFF
        try:
            self.send_command("APD:GATE?")
            mode_str = self.read_command()
            return SPCM50_GATINGMODE(int(mode_str))
        except Exception as ex:
            logger.error(f"GetGatingMode error: {ex}")
            return SPCM50_GATINGMODE.OFF

    def set_bin_length(self, bin_length_ms: float) -> bool:
        """
        If the device command expects the bin length in seconds,
        convert ms to seconds. For example, SPCM50 may use 'GATE:APERture' in seconds.
        """
        if not self.is_initialized:
            return False
        try:
            self.bin_length = bin_length_ms
            seconds_val = bin_length_ms / 1000.0
            self.send_command(f"GATE:APERture {seconds_val}")
            return True
        except Exception as ex:
            logger.error(f"SetBinLength error: {ex}")
            return False

    def get_bin_length(self) -> float:
        if not self.is_initialized:
            return 0.0
        try:
            self.send_command("GATE:APERture?")
            val_str = self.read_command()
            return float(val_str) * 1000.0  # Convert to ms
        except Exception as ex:
            logger.error(f"GetBinLength error: {ex}")
            return 0.0

    def set_time_between(self, delay_ms: float) -> bool:
        """
        Converts ms to seconds if the device parameter is in seconds.
        """
        if not self.is_initialized:
            return False
        try:
            seconds_val = delay_ms / 1000.0
            self.time_between_delay = delay_ms
            self.send_command(f"GATE:DELay {seconds_val}")
            return True
        except Exception as ex:
            logger.error(f"SetTimeBetween error: {ex}")
            return False

    def get_time_between(self) -> float:
        if not self.is_initialized:
            return 0.0
        try:
            self.send_command("GATE:DELay?")
            val_str = self.read_command()
            return float(val_str) * 1000.0
        except Exception as ex:
            logger.error(f"GetTimeBetween error: {ex}")
            return 0.0

    # ---------------------------
    # Helper Function
    # ---------------------------
    def _parse_counter_value(self, value_str: str) -> float:
        """
        Parses a string like "1234.56;XYZ" and returns the first numeric part as a float.
        Modify according to your protocol.
        """
        if not value_str:
            return 0.0
        try:
            parts = value_str.split(';')
            return float(parts[0])
        except:
            return 0.0
        
    def GetScanData(self) -> list:
        _list = []
        data = 0
        try:    
            avg_count = self.get_average_count()      
            data = avg_count
            print("SPCM50 Data: ", data)
            _list = [data]
            return _list     
        except Exception as e:
            _list = [data]
            print(e)
            return _list


from NMIXStudio.Container.NmiFrame import*
from NMIXStudio.Container.NmiContainer import NMIContainer
from NMIXStudio.Container.NmiContainer.enums import*

class NMIContainerBuilder():
    def __init__(self):
        self.container = NMIContainer()
    
    def SetWidthPixel(self,width):
        self.container.WidthPixel = width
        return self
    def SetHeightPixel(self,height):
        self.container.HeightPixel = height
        return self
    def SetRealHeight(self,height):
        self.container.RealHeight = height
        return self
    def SetRealWidth(self,width):
        self.container.RealWidth = width
        return self
    def SetRealHeightUnit(self,unit:ScaleUnit):
        self.container.RealHeightUnit = unit
        return self
    def SetRealWidthUnit(self,unit:ScaleUnit):
        self.container.RealWidthUnit = unit
        return self
    
    def SetRealHeightUnitPrefix(self,prefix:ScalePrefix):
        self.container.RealHeightUnitPrefix = prefix
        return self
    def SetRealWidthUnitPrefix(self,prefix:ScalePrefix):
        self.container.RealWidthUnitPrefix = prefix
        return self
    
    def AddImageChannel(self,channel:FrameChannels):
        self.container.AddNewImage(channel)
        return self

    def Build(self):
        return self.container


from .BuiltInImporter import*

def Singleton(cls):
    instances = {}
    def get_instance(*args, **kwargs):
        if cls not in instances:
            instances[cls] = cls(*args, **kwargs)
        return instances[cls]
    return get_instance

# Decorator 
def ExceptionLoggerAPIResponse(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            logging.error(f"Error occurred in {func.__name__}: {e}") 
            return None
    return wrapper


def RunInThread(func):
    def wrapper(*args, **kwargs):
        thread = threading.Thread(target=func, args=args, kwargs=kwargs)
        thread.start()
        return thread
    return wrapper